module.exports = {
    Panorama: {
        // Global variables to store configuration
        iFrameRequestUrl: null,
        panoramaContainer: null,
        iFrameElement: null,
        panoramaClickCheckBox: null,
        closeButton: null,
        mapClickEnabled: null,
        keyboardMovementEnabled: null,
        markerImage: null,
        // Variable to store whether the panorama is loaded or not
        panoramaLoaded: false,
        // Opens the panorama at the location x,y
        OpenPanoramaOnLocation: function (x, y) {
            // If panorama is closed
            if (this.panoramaContainer.style.display == 'none') {
                // Open panorama
                this.OpenPanoramaFrame();
            }
            // Then, open the coordinates
            this.OpenCoordinates(x, y);
        },
        // Opens the new coordinates on an already open panorama
        OpenCoordinates: function (x, y) {
            // Store this before timeout
            const panorama = this;
            // Wait until iframe is loaded
            if (panorama.panoramaLoaded == false) {
                timer = setTimeout(function () {
                    panorama.OpenCoordinates(x, y);
                }, 50);
            }
            else {
                // When the frame is loaded post message to open new coordinates
                panorama.iFrameElement.contentWindow.postMessage({ coordinates: [x, y] }, '*');
            }
        },
        // Opens the panorama iframe
        OpenPanoramaFrame: function () {
            // Fill in the iframes source
            if (this.iFrameElement.src != this.iFrameRequestUrl) {
                this.iFrameElement.src = this.iFrameRequestUrl;
            }
            // Unhide it
            this.panoramaContainer.style.display = 'initial';
            // Add this to close button and add an event listener
            this.closeButton.panorama = this;
            this.closeButton.addEventListener('click', this.HidePanoramaFrame);
        },
        // Hides the panorama frame
        HidePanoramaFrame: function (event) {
            // Hide container
            event.currentTarget.panorama.panoramaContainer.style.display = 'none';
            // Get the marker
            const marker = document.getElementById('view-cone');
            // If there is a marker
            if (marker != null) {
                // Remove marker
                marker.parentElement.removeChild(marker);
            }
        },
        // Listens to buttons and fires event accordingly
        MoveOnKeyPress: function () {
            // Store this before event
            const panorama = this;
            window.addEventListener('keydown', function (event) {
                // Initialize direction
                let direction;
                // If frame is loaded
                if (panorama.panoramaLoaded == true) {
                    // Fill in direction according to keys
                    if (event.key === 'ArrowUp' || event.key == 'w') {
                        direction = 'front';
                    }
                    else if (event.key == 'ArrowDown' || event.key == 's') {
                        direction = 'back';
                    }
                    else if (event.key == 'ArrowRight' || event.key == 'd') {
                        direction = 'right';
                    }
                    else if (event.key == 'ArrowLeft' || event.key == 'a') {
                        direction = 'left';
                    }
                    // If the key was valid then send a message including direction
                    if (direction != null) {
                        panorama.iFrameElement.contentWindow.postMessage({ direction: direction }, '*');
                    }
                }
            });
        },
        // Configure MapLibre map for panorama usage, call this method in map.onload
        ConfigureMapLibreMap: function (map, maplibregl) {
            // If keyboard movement is on
            if (this.keyboardMovementEnabled == true) {
                // Disable map keyboard controls
                map.keyboard.disable();
            }
            // Store this before events
            const panorama = this;
            // If map is clickable
            if (this.mapClickEnabled == true) {
                // Open panorama on click location
                map.on('click', function (e) {
                    // If checkbox is checked
                    if (panorama.panoramaClickCheckBox.checked == true) {
                        // Open panorama on location
                        panorama.OpenPanoramaOnLocation(e.lngLat.lng, e.lngLat.lat);
                    }
                })
            }
            // If marker is enabled (exists)
            if (panorama.markerImage != null) {
                // When marker gets update as message event
                window.addEventListener('message', function (event) {
                    if (event.data.angle != undefined && event.data.coordinates != undefined) {
                        if (panorama.panoramaContainer.style.display != 'none') {
                            // Get the marker
                            const marker = document.getElementById('view-cone');
                            // If there is a marker
                            if (marker != null) {
                                // Get its coordinates
                                const markerCoordinates = marker.dataset.coordinates.split(',').map(Number);
                                // If the new coordinates are different than the markers coordinates
                                if (event.data.coordinates[0] != markerCoordinates[0] || event.data.coordinates[1] != markerCoordinates[1]) {
                                    // Center map
                                    map.flyTo({ center: event.data.coordinates });
                                }
                                // Remove marker
                                marker.parentElement.removeChild(marker);
                            }
                            // Create new marker
                            const element = document.createElement('div');
                            element.id = 'view-cone';
                            element.dataset.coordinates = event.data.coordinates;
                            element.style = 'width:68px;height:150px;';
                            element.style.backgroundImage = "url(" + panorama.markerImage + ")";
                            new maplibregl.Marker(element).setLngLat(event.data.coordinates).setRotation(event.data.angle).addTo(map);
                        }
                    }
                })
            }
        },
        // Configure Panorama iFrameId is html id of the iframe, close button id is the html id of the close button, if marker is rendered on map markerId is the html id of the marker
        ConfigurePanorama: function (properties) {
            // Initialize global variables
            this.iFrameRequestUrl = properties.iFrameElementSrc;
            this.panoramaContainer = document.getElementById(properties.panoramaContainerId);
            this.iFrameElement = document.getElementById('panorama-iframe');
            this.closeButton = document.getElementById('panorama-close-button');
            this.panoramaClickCheckBox = document.getElementById(properties.panoramaCheckBoxElementId);
            this.mapClickEnabled = properties.mapClickEnabled;
            this.keyboardMovementEnabled = properties.keyboardControlsEnabled;
            this.markerImage = properties.markerImage;
            // Store this before events
            const panorama = this;
            // Catch the panorama loaded message
            window.addEventListener('message', function (event) {
                if (event.data.load == true) {
                    panorama.panoramaLoaded = true;
                }
            });
            // If configuration allows keyboard movement
            if (this.keyboardMovementEnabled == true) {
                // Add key events
                this.MoveOnKeyPress();
            }
        }
    }
}